cdef class SentiSynSet(object):

    cdef str __id
    cdef float __positiveScore, __negativeScore

    cpdef float getPositiveScore(self)
    cpdef float getNegativeScore(self)
    cpdef str getId(self)
    cpdef object getPolarity(self)


from SentiNet.PolarityType import PolarityType


cdef class SentiSynSet(object):

    def __init__(self, _id: str, positiveScore: float, negativeScore: float):
        """
        Constructor of SentiSynSet. Gets input id, positiveScore, negativeScore and sets all corresponding attributes.

        PARAMETERS
        ----------
        _id : str
            Id of the SentiSynSet.
        positiveScore : float
            Positive score of the SentiSynSet.
        negativeScore : float
            Negative score of the SentiSynSet.
        """
        self.__id = _id
        self.__positiveScore = positiveScore
        self.__negativeScore = negativeScore

    cpdef float getPositiveScore(self):
        """
        Accessor for the positiveScore attribute.

        RETURNS
        -------
        float
            PositiveScore of the SentiSynSet.
        """
        return self.__positiveScore

    cpdef float getNegativeScore(self):
        """
        Accessor for the negativeScore attribute.

        RETURNS
        -------
        float
            NegativeScore of the SentiSynSet.
        """
        return self.__negativeScore

    cpdef str getId(self):
        """
        Accessor for the id attribute.

        RETURNS
        -------
        str
            Id of the SentiSynSet.
        """
        return self.__id

    cpdef object getPolarity(self):
        """
        Returns the polarityType of the sentiSynSet. If the positive score is larger than the negative score, the
        polarity is positive; if the negative score is larger than the positive score, the polarity is negative; if
        both positive score and negative score are equal, the polarity is neutral.

        RETURNS
        -------
        PolarityType.pxd
            PolarityType of the sentiSynSet.
        """
        if self.__positiveScore > self.__negativeScore:
            return PolarityType.POSITIVE
        else:
            if self.__positiveScore < self.__negativeScore:
                return PolarityType.NEGATIVE
            else:
                return PolarityType.NEUTRAL

    def saveAsXml(self, outfile):
        """
        Method to write SynSets to the specified file in the XML format.

        PARAMETERS
        ----------
        outfile : file
            File to write XML files
        """
        outfile.write("<SYNSET>")
        outfile.write("<ID>" + self.__id + "</ID>")
        outfile.write("<PSCORE>" + self.__positiveScore.__str__() + "</PSCORE>")
        outfile.write("<NSCORE>" + self.__negativeScore.__str__() + "</NSCORE>")
        outfile.write("</SYNSET>\n")



import pkg_resources
from SentiNet.SentiLiteral cimport SentiLiteral
from SentiNet.PolarityType import PolarityType
import xml.etree.ElementTree


cdef class SentiLiteralNet(object):

    cdef dict __sentiLiteralList

    def __init__(self, fileName=None):
        """
        Constructor of Turkish SentiNet. Reads the turkish_sentinet.xml file from the resources directory. For each
        sentiSynSet read, it adds it to the sentiSynSetList.
        """
        cdef float positiveScore, negativeScore
        cdef str _name
        self.__sentiLiteralList = {}
        if fileName is None:
            fileName = pkg_resources.resource_filename(__name__, 'data/turkish_sentiliteralnet.xml')
        root = xml.etree.ElementTree.parse(fileName).getroot()
        for sentiLiteral in root:
            _name = ""
            positiveScore = 0.0
            negativeScore = 0.0
            for part in sentiLiteral:
                if part.tag == "NAME":
                    _name = part.text
                else:
                    if part.tag == "PSCORE":
                        positiveScore = float(part.text)
                    else:
                        negativeScore = float(part.text)
            if _name != "":
                self.__sentiLiteralList[_name] = SentiLiteral(_name, positiveScore, negativeScore)

    cpdef SentiLiteral getSentiLiteral(self, str _name):
        """
        Accessor for a single SentiLiteral.

        PARAMETERS
        ----------
        _name : str
            Id of the searched SentiLiteral.

        RETURNS
        -------
        SentiLiteral
            SentiLiteral with the given name.
        """
        return self.__sentiLiteralList[_name]

    cpdef list getPolarity(self, object polarityType):
        """
        Constructs and returns a list of ids, which are the ids of the SentiSynSets having polarity
        polarityType.

        PARAMETERS
        ----------
        polarityType : PolarityType.pxd
            PolarityTypes of the searched SentiSynSets

        RETURNS
        -------
        list
            A list of id having polarityType polarityType.
        """
        cdef list result
        cdef SentiLiteral sentiLiteral
        result = []
        for sentiLiteral in self.__sentiLiteralList.values():
            if sentiLiteral.getPolarity() == polarityType:
                result.append(sentiLiteral.getName())
        return result

    cpdef list getPositives(self):
        """
        Returns the ids of all positive SentiSynSets.

        RETURNS
        -------
        list
            A list of ids of all positive SentiSynSets.
        """
        return self.getPolarity(PolarityType.POSITIVE)

    cpdef list getNegatives(self):
        """
        Returns the ids of all negative SentiSynSets.

        RETURNS
        -------
        list
            A list of ids of all negative SentiSynSets.
        """
        return self.getPolarity(PolarityType.NEGATIVE)

    cpdef list getNeutrals(self):
        """
        Returns the ids of all neutral SentiSynSets.

        RETURNS
        -------
        list
            A list of ids of all neutral SentiSynSets.
        """
        return self.getPolarity(PolarityType.NEUTRAL)




import pkg_resources
from SentiNet.SentiSynSet cimport SentiSynSet
from SentiNet.PolarityType import PolarityType
import xml.etree.ElementTree


cdef class SentiNet(object):

    cdef dict __sentiSynSetList

    def __init__(self, fileName=None):
        """
        Constructor of Turkish SentiNet. Reads the turkish_sentinet.xml file from the resources directory. For each
        sentiSynSet read, it adds it to the sentiSynSetList.
        """
        cdef float positiveScore, negativeScore
        cdef str _id
        self.__sentiSynSetList = {}
        if fileName is None:
            fileName = pkg_resources.resource_filename(__name__, 'data/turkish_sentinet.xml')
        root = xml.etree.ElementTree.parse(fileName).getroot()
        for sentiSynSet in root:
            _id = ""
            positiveScore = 0.0
            negativeScore = 0.0
            for part in sentiSynSet:
                if part.tag == "ID":
                    _id = part.text
                else:
                    if part.tag == "PSCORE":
                        positiveScore = float(part.text)
                    else:
                        negativeScore = float(part.text)
            if _id != "":
                self.__sentiSynSetList[_id] = SentiSynSet(_id, positiveScore, negativeScore)

    cpdef SentiSynSet getSentiSynSet(self, str _id):
        """
        Accessor for a single SentiSynSet.

        PARAMETERS
        ----------
        _id : str
            Id of the searched SentiSynSet.

        RETURNS
        -------
        SentiSynSet
            SentiSynSet with the given id.
        """
        return self.__sentiSynSetList[_id]

    cpdef addSentiSynSet(self, SentiSynSet sentiSynSet):
        """
        Adds specified SentiSynSet to the SentiSynSet list.

        PARAMETERS
        ----------
        sentiSynSet : SentiSynSet
            SentiSynSet to be added
        """
        self.__sentiSynSetList[sentiSynSet.getId()] = sentiSynSet

    cpdef removeSentiSynSet(self, SentiSynSet sentiSynSet):
        """
        Removes specified SentiSynSet from the SentiSynSet list.

        PARAMETERS
        ----------
        sentiSynSet : SentiSynSet
            SentiSynSet to be removed
        """
        del self.__sentiSynSetList[sentiSynSet.getId()]

    cpdef list getPolarity(self, object polarityType):
        """
        Constructs and returns a list of ids, which are the ids of the SentiSynSets having polarity
        polarityType.

        PARAMETERS
        ----------
        polarityType : PolarityType.pxd
            PolarityTypes of the searched SentiSynSets

        RETURNS
        -------
        list
            A list of id having polarityType polarityType.
        """
        cdef list result
        cdef SentiSynSet sentiSynSet
        result = []
        for sentiSynSet in self.__sentiSynSetList.values():
            if sentiSynSet.getPolarity() == polarityType:
                result.append(sentiSynSet.getId())
        return result

    cpdef list getPositives(self):
        """
        Returns the ids of all positive SentiSynSets.

        RETURNS
        -------
        list
            A list of ids of all positive SentiSynSets.
        """
        return self.getPolarity(PolarityType.POSITIVE)

    cpdef list getNegatives(self):
        """
        Returns the ids of all negative SentiSynSets.

        RETURNS
        -------
        list
            A list of ids of all negative SentiSynSets.
        """
        return self.getPolarity(PolarityType.NEGATIVE)

    cpdef list getNeutrals(self):
        """
        Returns the ids of all neutral SentiSynSets.

        RETURNS
        -------
        list
            A list of ids of all neutral SentiSynSets.
        """
        return self.getPolarity(PolarityType.NEUTRAL)

    cpdef saveAsXml(self, str fileName):
        """
        Method to write SynSets to the specified file in the XML format.

        PARAMETERS
        ----------
        fileName : str
            file name to write XML files
        """
        cdef SentiSynSet synSet
        outfile = open(fileName, 'w', encoding='utf8')
        outfile.write("<SYNSETS>\n")
        for synSet in self.__sentiSynSetList.values():
            synSet.saveAsXml(outfile)
        outfile.write("</SYNSETS>\n")
        outfile.close()



from setuptools import setup

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()
from Cython.Build import cythonize

setup(
    ext_modules=cythonize(["SentiNet/*.pyx", "SentiNet/*.pxd"],
                          compiler_directives={'language_level': "3"}),
    name='NlpToolkit-SentiNet-Cy',
    version='1.0.6',
    packages=['SentiNet', 'SentiNet.data'],
    package_data={'SentiNet': ['*.pxd', '*.pyx', '*.c'],
                  'SentiNet.data': ['*.xml']},
    url='https://github.com/StarlangSoftware/TurkishSentiNet-Cy',
    license='',
    author='olcaytaner',
    author_email='olcay.yildiz@ozyegin.edu.tr',
    description='Turkish SentiNet',
    long_description=long_description,
    long_description_content_type='text/markdown',
)
